// TutorialView.cpp : implementation of the CTutorialView class
//

#include "stdafx.h"
#include "Tutorial.h"

#include "TutorialDoc.h"
#include "TutorialView.h"

#include "Cube.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTutorialView

IMPLEMENT_DYNCREATE(CTutorialView, CView)

BEGIN_MESSAGE_MAP(CTutorialView, CView)
	//{{AFX_MSG_MAP(CTutorialView)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_SIZE()
	ON_WM_PAINT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTutorialView construction/destruction

CTutorialView::CTutorialView()
{
	// TODO: add construction code here
	m_hGLContext = NULL;
	m_GLPixelIndex = 0;
	((CTutorialApp *)AfxGetApp())->pView = this;
}

CTutorialView::~CTutorialView()
{
}

BOOL CTutorialView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs
	cs.cx = 640;
	cs.cy = 480;
	cs.style |= (WS_CLIPCHILDREN | WS_CLIPSIBLINGS);
	cs.lpszClass = AfxRegisterWndClass(CS_OWNDC);

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CTutorialView drawing

void CTutorialView::OnDraw(CDC* pDC)
{
	CTutorialDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// TODO: add draw code for native data here
}

/////////////////////////////////////////////////////////////////////////////
// CTutorialView diagnostics

#ifdef _DEBUG
void CTutorialView::AssertValid() const
{
	CView::AssertValid();
}

void CTutorialView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CTutorialDoc* CTutorialView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CTutorialDoc)));
	return (CTutorialDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CTutorialView message handlers

BOOL CTutorialView::SetWindowPixelFormat(HDC hDC)
{
	PIXELFORMATDESCRIPTOR pixelDesc;

	pixelDesc.nSize		= sizeof(PIXELFORMATDESCRIPTOR);
	pixelDesc.nVersion	= 1;

	pixelDesc.dwFlags	=	PFD_DRAW_TO_WINDOW | 
							PFD_SUPPORT_OPENGL | 
							PFD_DOUBLEBUFFER |
							PFD_STEREO_DONTCARE;

	pixelDesc.iPixelType		= PFD_TYPE_RGBA;
	pixelDesc.cColorBits		= 32;
	pixelDesc.cRedBits			= 8;
	pixelDesc.cRedShift			= 16;
	pixelDesc.cGreenBits		= 8;
	pixelDesc.cGreenShift		= 8;
	pixelDesc.cBlueBits			= 8;
	pixelDesc.cBlueShift		= 0;
	pixelDesc.cAlphaBits		= 0;
	pixelDesc.cAlphaShift		= 0;
	pixelDesc.cAccumBits		= 64;	
	pixelDesc.cAccumRedBits		= 16;
	pixelDesc.cAccumGreenBits	= 16;
	pixelDesc.cAccumBlueBits	= 16;
	pixelDesc.cAccumAlphaBits	= 0;
	pixelDesc.cDepthBits		= 32;
	pixelDesc.cStencilBits		= 8;
	pixelDesc.cAuxBuffers		= 0;
	pixelDesc.iLayerType		= PFD_MAIN_PLANE;
	pixelDesc.bReserved			= 0;
	pixelDesc.dwLayerMask		= 0;
	pixelDesc.dwVisibleMask		= 0;
	pixelDesc.dwDamageMask		= 0;

	m_GLPixelIndex = ChoosePixelFormat(hDC, &pixelDesc);
	if (m_GLPixelIndex==0)
	{
		m_GLPixelIndex = 1;	// Let's choose a default index.
		if (DescribePixelFormat( hDC, m_GLPixelIndex, sizeof(PIXELFORMATDESCRIPTOR), &pixelDesc) == 0)
		{
			DWORD errorCode = GetLastError();
			CString errorStr;
			errorStr.Format("DescribePixelFormat returned error code %d.", errorCode);
			AfxMessageBox(errorStr);
			return FALSE;
		}
	}

	if (SetPixelFormat(	hDC, m_GLPixelIndex, &pixelDesc) ==FALSE)
	{
		DWORD errorCode = GetLastError();
		CString errorStr;
		errorStr.Format("SetPixelFormat returned error code %d.", errorCode);
		AfxMessageBox(errorStr);
		return FALSE;
	}
	return TRUE;
}

int CTutorialView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	// TODO: Add your specialized creation code here
	HWND hWnd = GetSafeHwnd();
	HDC hDC = ::GetDC(hWnd);

	if (SetWindowPixelFormat(hDC)==FALSE)
		return 0;

	if (CreateViewGLContext(hDC)==FALSE)
		return 0;

	return 0;
}
	
BOOL CTutorialView::CreateViewGLContext(HDC hDC)
{
	m_hGLContext = wglCreateContext(hDC);
	if (m_hGLContext == NULL)
	{
		DWORD errorCode = GetLastError();
		CString errorStr;
		errorStr.Format("wglCreateContext returned error code %d.", errorCode);
		AfxMessageBox(errorStr);
		return FALSE;
	}

	if (wglMakeCurrent(hDC, m_hGLContext)==FALSE)
	{
		DWORD errorCode = GetLastError();
		CString errorStr;
		errorStr.Format("wglCreateContext returned error code %d.", errorCode);
		AfxMessageBox(errorStr);
		return FALSE;
	}

	return TRUE;
}

void CTutorialView::OnDestroy() 
{
	if(wglGetCurrentContext()!=NULL) 
	{
		// make the rendering context not current
		wglMakeCurrent(NULL, NULL) ;
	}
	
	if (m_hGLContext!=NULL)
	{
		wglDeleteContext(m_hGLContext);
		m_hGLContext = NULL;
	}

	// Now the associated DC can be released.
	CView::OnDestroy();
	
	// TODO: Add your message handler code here
	
}

void CTutorialView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	
	// TODO: Add your message handler code here
	
	GLsizei width, height;
	GLdouble aspect;

	width = cx;
	height = cy;

	if (cy==0)
		aspect = (GLdouble)width;
	else
		aspect = (GLdouble)width/(GLdouble)height;

	glViewport(0, 0, width, height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45, aspect, 1, 10);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glDrawBuffer(GL_BACK);

	glEnable(GL_DEPTH_TEST);

	GLfloat	ambientProperties[]  = {0.8f, 0.8f, 0.08f, 1.0f};
	GLfloat positionProperties[] = {2.0f, 2.0f, -2.0f, 0.0f};
	
	glLightfv( GL_LIGHT0, GL_AMBIENT, ambientProperties);
	glLightfv( GL_LIGHT0, GL_POSITION, positionProperties);

	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHTING);

	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, ambientProperties);

	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		glEnable(GL_TEXTURE_2D);

		glShadeModel(GL_SMOOTH);
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	}

void CTutorialView::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	
	// TODO: Add your message handler code here
	static DWORD last_t = GetTickCount();
	DWORD t = GetTickCount() - last_t;
	if (t == 0)
		return;
	last_t = GetTickCount();


	glMatrixMode(GL_MODELVIEW);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	static CCube c(1, 0, 0, -8, 30, 35, 0, 0, 0, 0, 0.1, 0.02, -0.03);
	c.Draw(0, 0, 0);
	c.Update(t);

	glFlush();

	SwapBuffers(dc.m_ps.hdc);
	// Do not call CView::OnPaint() for painting messages
}
